import { csv } from 'd3';
import { select, pie, arc, scaleOrdinal, format } from 'd3';

// Colors for the sections in Pie Chart
const sectionColors = [
  'red',
  'blue',
  'yellow',
  'lime',
  'orange',
  'skyblue',
  'crimson',
  'wheat',
  'turquoise',
  'aqua',
];

// Data set URI
const csvUrl =
  'https://gist.githubusercontent.com/Mahesh-Polisetty/14316699380f4bc1beb720eb5ccd7fa9/raw/13f0454025771f8ed9d8deb145a5c6541b893fce/gistfile1.txt';

// Function to fetch data from the URI
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

// Select the SVG from the HTML body
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

// Format axes text
const xAxisTickFormat = (number) => format('.4~g')(number);

// Render Pie Chart on the canvas
const render = (data, total) => {
  const pieData = pie().value(
    (d) => d['2022 Revenue (USD)'],
  )(data);
  const colors = scaleOrdinal()
    .domain(data)
    .range(sectionColors);

  // Object for generating Arcs of the sections
  const arcGenerator = arc()
    .innerRadius(0)
    .outerRadius(250);

  // Segments are the partition in the Pie Chart
  const segments = arc()
    .innerRadius(0)
    .outerRadius(175)
    .padAngle(0)
    .padRadius(0);

  const sections = svg
    .append('g')
    .attr('transform', `translate(250,250)`)
    .selectAll('path')
    .data(pieData);

  sections
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d, i) => colors(i));

  // Render text on each section
  sections
    .enter()
    .append('text')
    .text(
      (d) =>
        xAxisTickFormat(
          d.data['2022 Revenue (USD)'] / 1000,
        ) + 'K',
    )
    .attr('transform', function (d) {
      return 'translate(' + arcGenerator.centroid(d) + ')';
    })
    .style('text-anchor', 'middle')
    .style('fill', 'black')
    .style('font-size', 15);

  // Render legend for the data
  const legends = svg
    .append('g')
    .attr('transform', 'translate(500,100)')
    .selectAll('.legends')
    .data(pieData);

  const legend = legends
    .enter()
    .append('g')
    .classed('.legends', true)
    .attr('transform', (d, i) => {
      return `translate(0,${(i + 1) * 30})`;
    });

  // Formats the legend for Revenue
  legend
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d, i) => colors(i));

  legend
    .append('text')
    .attr('x', 250)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text(
      (d) =>
        xAxisTickFormat(
          d.data['2022 Revenue (USD)'] / 1000,
        ) + 'K',
    );

  // Formats the legend for States
  legend
    .append('text')
    .attr('x', 25)
    .attr('y', 15)
    .attr('class', 'legend_text')
    .text((d) => d.data.State);
};

// Main function
const run = async () => {
  let data = await fetchData();
  let total = 0;
  data.forEach((d) => {
    d['2022 Revenue (USD)'] = +d['2022 Revenue (USD)'];
    total += d['2022 Revenue (USD)'];
  });
  render(data, total);
};
run();
